import DS from 'ember-data';
import JSONSerializer from '@ember-data/serializer/json';
import Store from '@ember-data/store';
import { CollectionReference, DocumentReference, Firestore } from 'firebase/firestore';
interface Links {
    [key: string]: string;
}
interface ResourceHash {
    id: string;
    links: Links;
    [key: string]: string | Links | CollectionReference;
}
interface RelationshipDefinition {
    key: string;
    type: string;
    options: {
        buildReference?(db: Firestore, record: unknown): CollectionReference;
    };
}
interface RelationshipSchema {
    name: string;
    kind: 'belongsTo' | 'hasMany';
    type: string;
    options: {
        async: boolean;
        polymorphic?: boolean;
        as?: string;
        inverse: string | null;
        [key: string]: unknown;
    };
}
interface AttributeSchema {
    name: string;
    kind?: 'attribute';
    options?: Record<string, unknown>;
    type?: string;
}
interface ModelClass {
    modelName: string;
    fields: Map<string, 'attribute' | 'belongsTo' | 'hasMany'>;
    attributes: Map<string, AttributeSchema>;
    relationshipsByName: Map<string, RelationshipSchema>;
    eachAttribute<T>(callback: (this: T, key: string, attribute: AttributeSchema) => void, binding?: T): void;
    eachTransformedAttribute<T>(callback: (this: T, key: string, relationship: RelationshipSchema) => void, binding?: T): void;
    determineRelationshipType(descriptor: {
        kind: string;
        type: string;
    }, store: Store): string;
    eachRelationship<T>(callback: (this: T, name: string, descriptor: RelationshipSchema) => void, binding?: T): void;
}
export default class CloudFirestoreSerializer extends JSONSerializer {
    extractRelationship(relationshipModelName: string, relationshipHash: DocumentReference): {
        id: string;
        type: string;
    } | {};
    extractRelationships(modelClass: ModelClass, resourceHash: ResourceHash): {};
    serializeBelongsTo(snapshot: DS.Snapshot, json: {
        [key: string]: string | null | DocumentReference;
    }, relationship: RelationshipDefinition): void;
    serialize(snapshot: DS.Snapshot, options: {}): {};
}
declare module 'ember-data/types/registries/serializer' {
    export default interface SerializerRegistry {
        'cloud-firestore-modular': CloudFirestoreSerializer;
    }
}
export {};
