import DS from 'ember-data';
import ModelRegistry from 'ember-data/types/registries/model';
import Service from '@ember/service';
import { CollectionReference, DocumentReference, DocumentSnapshot, Query, QuerySnapshot } from 'firebase/firestore';
interface QueryFetchConfig {
    modelName: keyof ModelRegistry;
    referenceKeyName: string;
    recordArray: DS.AdapterPopulatedRecordArray<unknown>;
    queryRef: Query;
    queryId?: string;
}
interface HasManyFetchConfig {
    modelName: keyof ModelRegistry;
    id: string;
    field: string;
    referenceKeyName: string;
    queryRef: Query;
}
export default class FirestoreDataManager extends Service {
    private store;
    private docListeners;
    private colListeners;
    private queryListeners;
    private hasManyListeners;
    willDestroy(): void;
    findRecordRealtime(modelName: keyof ModelRegistry, docRef: DocumentReference): Promise<DocumentSnapshot>;
    findAllRealtime(modelName: keyof ModelRegistry, colRef: CollectionReference): Promise<QuerySnapshot>;
    queryRealtime(config: QueryFetchConfig): Promise<DocumentSnapshot[]>;
    findHasManyRealtime(config: HasManyFetchConfig): Promise<DocumentSnapshot[]>;
    queryWithReferenceTo(queryRef: Query, referenceKey: string): Promise<DocumentSnapshot[]>;
    private setupDocRealtimeUpdates;
    private setupColRealtimeUpdates;
    setupQueryRealtimeUpdates(config: QueryFetchConfig, queryId: string): Promise<void>;
    setupHasManyRealtimeUpdates(config: HasManyFetchConfig, queryId: string): Promise<void>;
    private handleInitialDocRealtimeUpdates;
    private handleSubsequentDocRealtimeUpdates;
    private handleSubsequentColRealtimeUpdates;
    private handleInitialQueryRealtimeUpdates;
    private handleSubsequentQueryRealtimeUpdates;
    private handleInitialHasManyRealtimeUpdates;
    private handleSubsequentHasManyRealtimeUpdates;
    getReferenceToDoc(docSnapshot: DocumentSnapshot, modelName: keyof ModelRegistry, referenceKeyName: string, isRealtime?: boolean): Promise<DocumentSnapshot>;
    private pushRecord;
    private unloadRecord;
    private destroyListener;
}
declare module '@ember/service' {
    interface Registry {
        '-firestore-data-manager': FirestoreDataManager;
    }
}
export {};
